
from .loader import NetboxLoader

__title__ = 'prophetess-netbox'
__version__ = '0.2.4-2'
__description__ = 'Netbox plugin for Prophetess'
__author__ = 'Marco Ceppi'
__author_email__ = 'marco@vapor.io'
__url__ = 'https://github.com/vapor-ware/prophetess-netbox'
__license__ = 'GNU General Public License v3.0'
